# Molpy Up!
#
# Based on Max00355's version of DoodleJump, <https://github.com/Max00355/DoodleJump>
# Copyright (C) 2015 by Frankie Primerano,
# written in 120 minutes, see <https://www.reddit.com/r/gamedev/comments/3umhuq/doodle_jump_created_in_120_minutes_with_python/>.
#
# Copyright (C) 2018 by Peter Gerwinski <http://www.peter.gerwinski.de>,
# OTTified in about 120 minutes.
#
# This program is Free Software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License (GNU GPL), version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program, see the file LICENSE.  If not, see
# <http://www.gnu.org/licenses/>.
#
# The player figure "Molpy" is
# Copyright (C) 2013 by BlitzGirl, see user "BlitzGirl" at <http://forums.xkcd.com>,
# released under conditions compatible with both the GNU GPL v3 and the
# CC BY-NC 2.5 license (see below).
#
# All other artwork used by this program is based on xkcd 1190 "Time",
# Copyright (C) 2013 by Randall Munroe, see <http://xkcd.com/1190>,
# released under the Creative Commonns Attribution-NonCommercial Generic License,
# version 2.5 (CC BY-NC 2.5), see <https://creativecommons.org/licenses/by-nc/2.5/>.
#
# In particular, the contents of the directory "assets"
# is NOT subject to the GNU GPL.

import pygame
from pygame.locals import *
import sys
import random

class MolpyUp:
    def __init__(self):
        self.sx = 553
        self.sy = 395
        self.extrax = self.sx // 16
        self.screen = pygame.display.set_mode((self.sx, self.sy))
        pygame.font.init()
        self.font = pygame.font.SysFont("xkcd,sans", 25)
        self.xmovementmax = 10
        self.platformTypeSand = 0
        self.platformTypeGrass = 1
        self.platformTypeFloating = 2
        self.platformTypeBreaking = 3
        self.itemTypeAccelerator = 0
        self.itemTypeBeanie = 1
        self.itemTypeSandcastle = 2
        self.itemTypeFlag = 3
        self.imgPlatformSand = pygame.image.load("assets/platformSand.png").convert_alpha()
        self.imgPlatformGrass = pygame.image.load("assets/platformGrass.png").convert_alpha()
        self.imgPlatformFloating = pygame.image.load("assets/platformFloating.png").convert_alpha()
        self.imgPlatformBreaking = pygame.image.load("assets/platformBreaking.png").convert_alpha()
        self.imgPlatformBroken = pygame.image.load("assets/platformBroken.png").convert_alpha()
        self.imgPlayerRightUp = pygame.image.load("assets/playerRightUp.png").convert_alpha()
        self.imgPlayerRightDown = pygame.image.load("assets/playerRightDown.png").convert_alpha()
        self.imgPlayerLeftUp = pygame.image.load("assets/playerLeftUp.png").convert_alpha()
        self.imgPlayerLeftDown = pygame.image.load("assets/playerLeftDown.png").convert_alpha()
        self.imgPlayerRightUpBeanie = pygame.image.load("assets/playerRightUpBeanie.png").convert_alpha()
        self.imgPlayerRightDownBeanie = pygame.image.load("assets/playerRightDownBeanie.png").convert_alpha()
        self.imgPlayerLeftUpBeanie = pygame.image.load("assets/playerLeftUpBeanie.png").convert_alpha()
        self.imgPlayerLeftDownBeanie = pygame.image.load("assets/playerLeftDownBeanie.png").convert_alpha()
        self.imgAccelerator = pygame.image.load("assets/accelerator.png").convert_alpha()
        self.imgAcceleratorUsed = pygame.image.load("assets/acceleratorUsed.png").convert_alpha()
        self.imgBeanie = pygame.image.load("assets/beanie.png").convert_alpha()
        self.imgSandcastle = pygame.image.load("assets/sandcastle.png").convert_alpha()
        self.imgFlagLeft = pygame.image.load("assets/flagLeft.png").convert_alpha()
        self.imgFlagLeftDown = pygame.image.load("assets/flagLeftDown.png").convert_alpha()
        self.imgFlagRight = pygame.image.load("assets/flagRight.png").convert_alpha()
        self.imgFlagRightDown = pygame.image.load("assets/flagRightDown.png").convert_alpha()
    
    def reset(self):
        self.cameray = 0
        self.score = 0
        self.sandcastles = 0
        self.speed = 30
        self.items = []
        self.playerx = self.sx // 2
        self.playery = self.sy * 3 // 4
        self.direction = 0
        self.jump = 0
        self.gravity = 0
        self.xmovement = 0
        self.playerWearsBeanie = False
        self.playerCarriesFlag = False
        self.platforms = [[self.playerx, self.playery + self.sy // 6, self.platformTypeSand, False]]
        self.generatePlatforms()

    def updatePlayer(self):
        if not self.jump:        
            self.playery += self.gravity
            self.gravity += 1
        elif self.jump:
            self.playery -= self.jump
            self.jump -= 1
        key = pygame.key.get_pressed()
        if key[K_RIGHT]:
            if self.xmovement < self.xmovementmax:
                self.xmovement += 1
            self.direction = 0
        elif key[K_LEFT]:
            if self.xmovement > -self.xmovementmax:
                self.xmovement -= 1
            self.direction = 1
        elif key[K_PLUS]:
            self.speed += 1
            self.screen.blit(self.font.render("Speed: " + str(self.speed), -1, (0, 0, 0)), (self.sx // 32, self.sy - self.sy // 12))
        elif key[K_MINUS]:
            self.speed -= 1
            self.screen.blit(self.font.render("Speed: " + str(self.speed), -1, (0, 0, 0)), (self.sx // 32, self.sy - self.sy // 12))
        else:
            if self.xmovement > 0:
                self.xmovement -= 1
            elif self.xmovement < 0:
                self.xmovement += 1
        if self.playerx > self.sx + self.extrax:
            self.playerx = -self.extrax
        elif self.playerx < -self.extrax:
            self.playerx = self.sx + self.extrax
        self.playerx += self.xmovement
        if self.playery - self.cameray <= self.sy // 3:
            self.cameray -= self.sy // 60
        if not self.direction:
            if self.jump:
                if self.playerWearsBeanie:
                    self.screen.blit(self.imgPlayerRightDownBeanie, (self.playerx, self.playery - self.cameray))
                else:
                    self.screen.blit(self.imgPlayerRightDown, (self.playerx, self.playery - self.cameray))
                if self.playerCarriesFlag:
                    self.screen.blit(self.imgFlagLeft, (self.playerx + 24, self.playery - 10 - self.cameray))
            else:
                if self.playerWearsBeanie:
                    self.screen.blit(self.imgPlayerRightUpBeanie, (self.playerx, self.playery - self.cameray))
                else:
                    self.screen.blit(self.imgPlayerRightUp, (self.playerx, self.playery - self.cameray))
                if self.playerCarriesFlag:
                    self.screen.blit(self.imgFlagLeftDown, (self.playerx + 22, self.playery - 10 - self.cameray))
        else:
            if self.jump:
                if self.playerWearsBeanie:
                    self.screen.blit(self.imgPlayerLeftDownBeanie, (self.playerx, self.playery - self.cameray))
                else:
                    self.screen.blit(self.imgPlayerLeftDown, (self.playerx, self.playery - self.cameray))
                if self.playerCarriesFlag:
                    self.screen.blit(self.imgFlagRight, (self.playerx + 13, self.playery - 10 - self.cameray))
            else:
                if self.playerWearsBeanie:
                    self.screen.blit(self.imgPlayerLeftUpBeanie, (self.playerx, self.playery - self.cameray))
                else:
                    self.screen.blit(self.imgPlayerLeftUp, (self.playerx, self.playery - self.cameray))
                if self.playerCarriesFlag:
                    self.screen.blit(self.imgFlagRightDown, (self.playerx + 11, self.playery - 10 - self.cameray))

    def updatePlatforms(self):
        for p in self.platforms:
            rect = pygame.Rect(p[0], p[1], self.imgPlatformSand.get_width() - 10, self.imgPlatformSand.get_height())
            player = pygame.Rect(self.playerx, self.playery, self.imgPlayerRightUp.get_width() - 10, self.imgPlayerRightUp.get_height())
            if rect.colliderect(player) and self.gravity and self.playery < (p[1] - self.cameray):
                if p[2] != self.platformTypeBreaking:
                    self.jump = self.sy // 20
                    self.gravity = 0
                else:
                    p[-1] = 1
            if p[2] == self.platformTypeFloating:
                if p[-1] == 1:
                    p[0] += self.extrax // 10
                    if p[0] + self.imgPlatformFloating.get_width() > self.sx + self.extrax:
                        p[-1] = 0
                else:
                    p[0] -= self.extrax // 10
                    if p[0] <= 0:
                        p[-1] = 1

    def choosePlatformType(self):
        platformType = random.randint(0, 1000)
        if platformType < 800:
            if platformType < self.score:
                return self.platformTypeGrass
            else:
                return self.platformTypeSand
        elif platformType < 900:
            return self.platformTypeFloating
        else:
            return self.platformTypeBreaking

    def drawPlatforms(self):
        for p in self.platforms:
            check = self.platforms[1][1] - self.cameray
            if check > self.sy:
                platformType = self.choosePlatformType()
                self.platforms.append([random.randint(0, self.sx * 7 // 8), self.platforms[-1][1] - self.sy // 12, platformType, False])
                check = random.randint(0, 1000)
                if check > 900 and platformType <= self.platformTypeGrass:
                    x = self.platforms[-1][0]
                    y = self.platforms[-1][1]
                    itemType = random.randint(0,1000)
                    if itemType < 100:
                        itemType = self.itemTypeBeanie
                        self.items.append([x + self.imgPlatformSand.get_width() * 3 // 4, y + 6, itemType, False])
                    elif itemType < 300 and platformType == self.platformTypeSand:
                        itemType = self.itemTypeSandcastle
                        self.items.append([x + self.imgPlatformSand.get_width() // 4, y + 6, itemType, False])
                    elif itemType < 800 and platformType == self.platformTypeSand:
                        itemType = self.itemTypeFlag
                        self.items.append([x + self.imgPlatformSand.get_width() // 2, y + 6, itemType, False])
                    else:
                        itemType = self.itemTypeAccelerator
                        self.items.append([x, y + 10, itemType, False])
                self.platforms.pop(0)
                self.score += 1
            if p[2] == self.platformTypeSand:
                self.screen.blit(self.imgPlatformSand, (p[0], p[1] - self.cameray))
            elif p[2] == self.platformTypeGrass:
                self.screen.blit(self.imgPlatformGrass, (p[0], p[1] - self.cameray))
            elif p[2] == self.platformTypeFloating:
                self.screen.blit(self.imgPlatformFloating, (p[0], p[1] - self.cameray))
            elif p[2] == self.platformTypeBreaking:
                if not p[3]:
                    self.screen.blit(self.imgPlatformBreaking, (p[0], p[1] - self.cameray))
                else:
                    self.screen.blit(self.imgPlatformBroken, (p[0], p[1] - self.cameray))
        for item in self.items:
            if item[2] == self.itemTypeAccelerator:
                if item[-1]:
                    imgItem = self.imgAcceleratorUsed
                else:
                    imgItem = self.imgAccelerator
            elif item[2] == self.itemTypeBeanie:
                if not item[-1]:
                    imgItem = self.imgBeanie
                else:
                    imgItem = None
            elif item[2] == self.itemTypeSandcastle:
                imgItem = self.imgSandcastle
                if item[-1]:
                    self.screen.blit(self.imgFlagRight, (item[0] + 4, item[1] - imgItem.get_height() + 2 - self.imgFlagRight.get_height() - self.cameray))
            elif item[2] == self.itemTypeFlag:
                if not item[-1]:
                    imgItem = self.imgFlagRight
                else:
                    imgItem = None
            if imgItem:
                self.screen.blit(imgItem, (item[0], item[1] - imgItem.get_height() - self.cameray))
            if not item[-1]:
                if pygame.Rect(item[0], item[1] - imgItem.get_height(), imgItem.get_width(), imgItem.get_height()).colliderect(pygame.Rect(self.playerx, self.playery, self.imgPlayerRightUp.get_width(), self.imgPlayerRightUp.get_height())):
                    if item[2] == self.itemTypeAccelerator:
                        self.jump = self.sy // 12
                        self.cameray -= self.sy // 12
                        item[-1] = True
                    elif item[2] == self.itemTypeBeanie and not self.playerWearsBeanie:
                        self.playerWearsBeanie = True
                        item[-1] = True
                    elif item[2] == self.itemTypeSandcastle and self.playerCarriesFlag:
                        self.playerCarriesFlag = False
                        item[-1] = True
                        self.sandcastles += 1
                    elif item[2] == self.itemTypeFlag and not self.playerCarriesFlag:
                        self.playerCarriesFlag = True
                        item[-1] = True

    def generatePlatforms(self):
        on = self.sy
        while on > -self.sy // 6:
            x = random.randint(0,self.sx * 8 // 7)
            self.platforms.append([x, on, self.choosePlatformType(), 0])
            on -= self.sy // 12

    def drawGrid(self):
        for x in range(80):
            pygame.draw.line(self.screen, (222,222,222), (x * 12, 0), (x * 12, self.sy))
            pygame.draw.line(self.screen, (222,222,222), (0, x * 12), (self.sx, x * 12))

    def waitForIt(self, its):
        run = False
        while not run:
            event = pygame.event.poll()
            if event.type == KEYDOWN:
                key = pygame.key.get_pressed()
                if key[K_ESCAPE]:
                    sys.exit()
                for it in its:
                    if key[it]:
                        return it

    def showScore(self):
        line1 = self.font.render("Press ENTER to restart,", -1, (0, 0, 0))
        line2 = self.font.render("ESC to quit.", -1, (0, 0, 0))
        w = max(line1.get_width(), line2.get_width())
        h = line1.get_height() + line2.get_height()
        dx = 10
        dy = 5
        pygame.draw.rect(self.screen, (255,255,255), ((self.sx - w) // 2 - dx, (self.sy - h) // 2 - dy, w + 2 * dx, h + 2 * dy))
        pygame.draw.rect(self.screen, (0,0,0), ((self.sx - w) // 2 - dx, (self.sy - h) // 2 - dy, w + 2 * dx, h + 2 * dy), 3)
        self.screen.blit(line1, ((self.sx - line1.get_width()) // 2, self.sy // 2 - line1.get_height()))
        self.screen.blit(line2, ((self.sx - line2.get_width()) // 2, self.sy // 2))
        pygame.display.flip() 
        self.waitForIt([K_RETURN])

    def run(self):
        clock = pygame.time.Clock()
        self.reset()
        while True:
            self.screen.fill((255,255,255))
            clock.tick(self.speed)
            for event in pygame.event.get():
                if event.type == QUIT:
                    sys.exit()
                elif event.type == KEYDOWN:
                    key = pygame.key.get_pressed()
                    if key[K_ESCAPE]:
                        sys.exit()
                    elif key[K_SPACE]:
                        self.waitForIt([K_SPACE])
            self.drawGrid()
            self.drawPlatforms()
            self.updatePlayer()
            self.updatePlatforms()
            self.screen.blit(self.font.render(str(self.score), -1, (0, 0, 0)), (self.sx // 32, self.sy // 24))
            for i in range(0, self.sandcastles):
                self.screen.blit(self.imgFlagRight, (self.sx // 32 + 10 * i, self.sy // 8))
            if self.playery - self.cameray > self.sy * 7 // 6:
                self.showScore()
                self.reset()
            pygame.display.flip() 

MolpyUp().run()
